"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileBSON = void 0;
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
const prettier_1 = __importDefault(require("prettier"));
const options_1 = require("./options");
function format(text, options) {
    return prettier_1.default.format(text, Object.assign(Object.assign({}, options), { parser: "typescript" }));
}
/**
 * JSON Schema types ("type" keyword)
 * https://json-schema.org/understanding-json-schema/reference/type.html
 *
 * string
 * number
 * object
 * array
 * boolean
 * null
 *
 * BSON types ("bsonType" keyword)
 * https://docs.mongodb.com/manual/reference/operator/query/jsonSchema/#available-keywords
 * https://docs.mongodb.com/manual/reference/operator/query/type/#document-type-available-types
 *
 * [X] number
 * [ ] double
 * [X] string
 * [ ] object
 * [ ] array
 * [ ] binData
 * [ ] undefined  -- Deprecated
 * [ ] objectId
 * [X] bool
 * [X] date
 * [X] null
 * [ ] regex
 * [ ] dbPointer  -- Deprecated
 * [ ] javascript
 * [ ] symbol     -- Deprecated
 * [ ] javascriptWithScope
 * [X] int
 * [ ] timestamp
 * [ ] long
 * [X] decimal
 * [ ] minKey
 * [ ] maxKey
 *
 * BSON types -> TS
 */
const bsonToTs = new Map([
    ["number", "number"],
    ["string", "string"],
    ["bool", "boolean"],
    ["date", "Date"],
    ["null", "null"],
    ["int", "number"],
    ["decimal", "Decimal128"],
]);
/**
 * Generate the 'tsType' field for bson types.
 * https://github.com/bcherny/json-schema-to-typescript#custom-schema-properties
 */
function buildTsType(bsonType) {
    // Handle array of BSON types
    //
    // This is necessary since
    // https://github.com/bcherny/json-schema-to-typescript does not support
    // 'tsType' as an array
    if (Array.isArray(bsonType)) {
        return bsonType.map((e) => buildTsType(e)).join("|");
    }
    if (typeof bsonType !== "string")
        return null;
    // Return the 'tsType' based on the BSON type
    return bsonToTs.get(bsonType) || null;
}
/**
 * Recursively annotates the schema with "tsType" properties based on the
 * "bsonType" field
 */
function setTsType(schema) {
    if (Array.isArray(schema)) {
        return schema.map(setTsType);
    }
    if (typeof schema === "object" && schema !== null) {
        const bsonType = "bsonType" in schema ? schema.bsonType : undefined;
        const isEnum = Reflect.has(schema, "enum");
        const tsType = buildTsType(bsonType);
        // Add 'tsType' fields to objects, except enums
        const v = !isEnum && tsType ? Object.assign(Object.assign({}, schema), { tsType }) : schema;
        return Object.entries(v).reduce((acc, [key, value]) => {
            return Object.assign(Object.assign({}, acc), { [key]: setTsType(value) });
        }, {});
    }
    return schema;
}
/**
 * Recursively checks if a schema relies on the Decimal128 BSON type anywhere
 */
function hasDecimal128(schema) {
    if (schema === null || typeof schema !== "object") {
        return false;
    }
    if (Array.isArray(schema)) {
        return schema.some(hasDecimal128);
    }
    return Object.entries(schema).some(([key, value]) => {
        if (key === "bsonType") {
            if (value === "decimal")
                return true;
            if (Array.isArray(value))
                return value.includes("decimal");
        }
        if (typeof value === "object")
            return hasDecimal128(value);
        return false;
    });
}
function compileBSON(schema, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const baseBanner = (options === null || options === void 0 ? void 0 : options.bannerComment) || options_1.defaultOptions.bannerComment;
        // Import Decimal128 if the type annotations use it
        const bannerComment = (hasDecimal128(schema)
            ? [...baseBanner, `import { Decimal128 } from "bson";`]
            : baseBanner).join("\n");
        const compileJSONOptions = {
            bannerComment,
            enableConstEnums: options === null || options === void 0 ? void 0 : options.enableConstEnums,
            ignoreMinAndMaxItems: options === null || options === void 0 ? void 0 : options.ignoreMinAndMaxItems,
            strictIndexSignatures: options === null || options === void 0 ? void 0 : options.strictIndexSignatures,
            unknownAny: options === null || options === void 0 ? void 0 : options.unknownAny,
        };
        // Add "tsType" fields as needed
        const newSchema = setTsType(schema);
        // Generate types
        const output = yield json_schema_to_typescript_1.compile(newSchema, "", compileJSONOptions);
        return format(output, options === null || options === void 0 ? void 0 : options.prettier);
    });
}
exports.compileBSON = compileBSON;
//# sourceMappingURL=compile.js.map